# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from typing import Literal

ConfiguracaoEventoTipo = Literal[
    'VENCIMENTO',
    'DESCONTO',
    'INFORMATIVO_MAIS',
    'INFORMATIVO_MENOS',
]

ConfiguracaoEventoClassificacao = Literal[
    'AVPREINDUMDOZE',
    'ADI13SAL',
    'AVPREIND13SAL',
    'INTEG13SAL',
    'RPPSABAT13',
    'RPPSABAT',
    'ABATSALM13',
    'ABATSALM',
    'ADIFERIAS',
    'ADISALNDESC',
    'AUXNAT',
    'AVPREDESC',
    'AVPREIND',
    'BASABATINSSDV',
    'BASINSSDV',
    'BASINSSDV13SAL',
    'BASINSSOUTEMP13SAL',
    'BASINSSOUTEMP',
    'BASIRRFDV',
    'BASIRRFDV13SAL',
    'BASIRRFFERIASANT',
    'BASIRRFFERIASRESCDV',
    'BASIRRFOUTEMP13SAL',
    'BASIRRFOUTEMP',
    'DESCHAUXMAT13SAL',
    'DESCHAUXMAT',
    'DESC13SALADI',
    'DESCADISAL',
    'DESC13SALDEVREINTG',
    'DESCDEP',
    'DEVINSS',
    'DEVIRRF',
    'DEVFGTS',
    'EMPRESTIMO',
    'FALTADSR',
    'FALTAS',
    'FGTS13SAL',
    'FGTSAVPRE',
    'FGTSMESANT',
    'FGTSSOBFERIAS',
    'FGTS',
    'FUNDASS13SAL',
    'FUNDASS',
    'FUNDFIN13SAL',
    'FUNDFIN',
    'FUNDPREV13SAL',
    'FUNDPREV',
    'HAUXMATNPAG',
    'INSS13SAL',
    'INSSSOBHAUXMATNPAG',
    'INSS',
    'IRRF13SAL',
    'IRRFFERIASRESC',
    'IRRF',
    'ISS',
    'MULTAFGTS',
    'NENHUMA',
    'PARCISENIRRFMOLGRVFERIASADI',
    'PARCISENIRRFMOLGRVFERIASRESC',
    'PARCISENFUNDFIN13SAL',
    'PARCISENFUNDFIN',
    'PARCISENFUNDPREV13SAL',
    'PARCISENFUNDPREV',
    'PARCISENIRRF13SAL',
    'PARCISENIRRFMOLGRV13SAL',
    'PARCISENIRRFMOLGRVFERIAS',
    'PARCISENIRRFMOLGRV',
    'PARCISENIRRFSOBFERIASRESC',
    'PARCISENIRRF',
    'PENSJUDDESC',
    'PLANOSAUDE',
    'PREVESTDECSAL',
    'PREVEST',
    'PROCEDMED',
    'PONTIDAO',
    'PRORROGAUXMAT11770',
    'REDBASINSSTRANSPAUT',
    'REDBASIRRFTRANSPAUT',
    'SALFAMAPOS',
    'SALFAMCEL',
    'SALFAMESP',
    'SALFAMEST',
    'SALFAMPAGMAIOR',
    'SENAT',
    'SEST',
    'SOBREAVISO',
    'VALETRANSP',
    'HORASFERIAS',
    'FERIASADI',
    'HORASFERIASCOL',
    'HORASFERIASAVPREVIND',
    'FERIASTERCO',
    'ABONOTERCO',
    'FERDOBROTERCO',
    'FERINDENTERCO',
    'FERPROPTERCO',
    'FERVENCTERCO',
    'BASEINSSNEG',
    'AUXDOENCA15DIAS',
    'ABATINSSFER',
    'ABATFUNDOPREVIDENCIAFER',
    'ABATPREVIDENCIAESTADUALFER',
    'ABATFUNDOFINANCEIROFER',
    'ABATFUNDOASSISTENCIAFER',
    'LIQ13SALINTEGADI',
    'TOTDESC13SALINTEGADI',
    'DESC13SALINTEGADI',
    'DESC13SALINTEGADIESOCIAL',
    'JUROSMORA',
    'AUXMATERNIDADE13',
    'PENSAOALIMENTICIA',
    'BASIRRFRRA',
    'DEDUCSIMPLIRRFMENSAL',
    'DEDUCCOMPLIRRFMENSAL',
    'DEDUCSIMPLIRRF13',
    'DEDUCCOMPLIRRF13',
    'BASINSSOUTEMPAUTONOMO13SAL',
    'BASINSSOUTEMPAUTONOMO',
    'PREVCOMPL',
    'PREVCOMPL13SAL',
    'REEMBDESPSAUDE'
]

FolhaSituacao = Literal[
    'PENDENTE',
    'CALCULADA',
    'FECHADA',
    'ERRO',
    'LANCADA'
]

FolhaTipoProcessamento = Literal[
    'MENSAL',
    'DECIMO_TERCEIRO_SALARIO',
    'RESCISAO',
    'FERIAS',
    'PAGAMENTO_ANTERIOR'
]

FolhaSubTipoProcessamento = Literal[
    'ADIANTAMENTO',
    'INTEGRAL',
    'COMPLEMENTAR'
]

TipoBaseClassificacao = Literal[
    'ABATIMENTO_INSS_13_SALARIO',
    'ABATIMENTO_INSS_FERIAS',
    'ABATIMENTO_INSS',
    'ABATIMENTO_IRRF_13_SALARIO',
    'ABATIMENTO_IRRF_FERIAS',
    'ABATIMENTO_IRRF',
    'BASE_MARGEM_CONSIGNAVEL',
    'COMPOEM_HORAS_DO_MES',
    'CONTRIB_SINDICAL_ANUAL',
    'DESC_13_SALARIO_REINTEGRACAO',
    'DESCONTO_IRRF_13_SALARIO',
    'DESCONTO_IRRF_FERIAS_RESC',
    'DESCONTO_IRRF_FERIAS',
    'DESCONTO_IRRF',
    'DESCONTO_UM_TERCO_DE_FERIAS',
    'EXCEDENTE_INSS_13_SALARIO',
    'EXCEDENTE_INSS_FERIAS',
    'EXCEDENTE_INSS',
    'EXCESSO_DA_MARGEM_CONSIGNAVEL',
    'FGTS_13_SALARIO',
    'FGTS_AVISO_PREVIO',
    'FGTS_FERIAS',
    'FGTS',
    'FUNDO_ASSISTENCIA_13_SALARIO',
    'FUNDO_ASSISTENCIA_FERIAS',
    'FUNDO_ASSISTENCIA',
    'FUNDO_FINANCEIRO_13_SALARIO',
    'FUNDO_FINANCEIRO_FERIAS',
    'FUNDO_FINANCEIRO',
    'FUNDO_PREVIDENCIA_13_SALARIO',
    'FUNDO_PREVIDENCIA_FERIAS',
    'FUNDO_PREVIDENCIA',
    'HORAS_EXTRAS',
    'INSS_13_SALARIO',
    'INSS_FERIAS',
    'INSS_OUTRAS_EMP_13_SAL',
    'INSS_OUTRAS_EMPRESAS',
    'INSS',
    'IRRF_13_SALARIO',
    'IRRF_FERIAS_RESCISAO',
    'IRRF_FERIAS',
    'IRRF_OUTRAS_EMP_13_SAL',
    'IRRF_OUTRAS_EMPRESAS',
    'IRRF',
    'MEDIA_AUXILIO_MATERNIDADE_PROPORCIONAL',
    'MEDIA_AUXILIO_MATERNIDADE',
    'OUTRAS_BASES',
    'PAGA_PROPORCIONAL',
    'PERICULOSIDADE',
    'PREV_ESTADUAL_13_SALARIO',
    'PREV_ESTADUAL_FERIAS',
    'PREV_ESTADUAL',
    'SALARIO_BASE',
    'SALARIO_FAMILIA_NORMAL',
    'SALDO_DA_MARGEM_CONSIGNAVEL',
    'SINDICATO',
    'UM_TERCO_FERIAS_VENCIDAS_RESC',
    'VALOR_DA_MARGEM_CONSIGNAVEL',
    'DEVOLUCAO_IRRF',
    'DEVOLUCAO_INSS',
    'IRRF_RRA',
    'DEDUC_IRRF_MENSAL',
    'DEDUC_IRRF_13',
    'INSS_OUTRAS_EMP_AUTONOMO_13_SAL',
    'INSS_OUTRAS_EMPRESAS_AUTONOMO'
]

MatriculaTipo = Literal[
    'FUNCIONARIO',
    'ESTAGIARIO',
    'AUTONOMO',
    'APOSENTADO',
    'PENSIONISTA',
]

ConcursoProcessoSeletivoProcesso = Literal[
    'COMPLETO',
    'SIMPLIFICADO',
    'NULO'
]

ConcursoProcessoSeletivoRegimeJuridico = Literal[
    'CLT',
    'ESTATUTARIO',
    'MISTO',
    'NULO'
]

ConcursoProcessoSeletivoResponsavelExecucao = Literal[
    'OUTRA_UNIDADE_GESTORA',
    'PRESTADORA_SERVICO_COM_CONTRATO',
    'PRESTADORA_SERVICO_SEM_CONTRATO',
    'PROPRIA_UNIDADE_GESTORA',
    'NULO'
]

ConcursoProcessoSeletivoSituacaoConcursoProcessoSeletivo = Literal[
    'EM_ABERTO',
    'ENCERRADO',
    'INSCRICOES_ABERTAS',
    'INSCRICOES_FECHADAS',
    'PERIODO_CLASSIFICATORIO',
    'PERIODO_CONVOCATORIO',
    'REALIZACAO_AVALIACOES',
    'NULO'
]

ConcursoProcessoSeletivoTipoAvaliacao = Literal[
    'PROVAS',
    'PROVAS_TITULOS',
    'TITULOS',
    'NULO'
]

ConcursoProcessoSeletivoTipoProcessoAvaliacao = Literal[
    'AUTOMATICO',
    'MANUAL',
    'NULO'
]

ConcursoProcessoSeletivoTipoRecrutamento = Literal[
    'CONCURSO_PUBLICO',
    'PROCESSO_SELETIVO',
    'NULO'
]

TipoAtoClassificacao = Literal[
    'ATA',
    'ATA_POSSE',
    'ATO',
    'ATO_ADMINISTRATIVO',
    'ATO_COMISSAO_EXEC_LEGIS',
    'ATO_CONSELHO_DELIBERATIVO',
    'ATO_CONSELHO_FISCAL',
    'ATO_CONSORCIO',
    'COMUNICACAO_INTERNA',
    'CONCESSAO_AFAST_INTER_PARTICULAR',
    'CONSTITUICAO',
    'CONTRATO',
    'CONVENIO',
    'DECRETO',
    'DECRETO_LEGISLATIVO',
    'DECRETO_LEI',
    'DELIBERACAO',
    'DESPACHO',
    'EDITAL',
    'EMENDA',
    'EMENDA_CONSTITUCIONAL',
    'EMENDA_MODIFICATIVA',
    'ESTATUTO_SOCIAL',
    'IMAGEM_EXEMPLAR_PUBLICA_ORGAO_OFICIAL',
    'INSTRUCAO',
    'JUSTIFICATIVA_CANCELAMENTO_INTERVENCAO',
    'LEI',
    'LEI_AUTORIZATIVA',
    'LEI_COMPLEMENTAR',
    'LEI_CRIACAO',
    'LEI_MUNICIPAL',
    'LEI_ORDINARIA',
    'LEI_ORGANICA',
    'MEDICAO',
    'MEDIDA_PROVISORIA',
    'MEMORANDO',
    'MULTIMIDIA',
    'OFICIO',
    'ORCAMENTO',
    'ORCAMENTO_BASE',
    'PARECER',
    'PLANILHA_ORCAMENTARIA_ADITIVO',
    'PLANILHA_ORCAMENTARIA_CONTRATADA',
    'PORTARIA',
    'PROCESSO',
    'PROCESSO_JUDICIAL',
    'PROJETO',
    'REGIMENTO_INTERNO',
    'REGISTRO_IMOVEIS',
    'REQUERIMENTO',
    'RESOLUCAO',
    'TERMO_MEDICAO',
    'TERMO_PARALISACAO',
    'TERMO_RECEBIMENTO'
]

MatriculaSituacao = Literal[
    'TRABALHANDO',
    'FALTA',
    'AFASTADO',
    'CEDIDO',
    'SUSPENSO',
    'FERIAS',
    'DEMITIDO',
    'SEM_VINCULO',
    'APOSENTADO',
    'PENSIONISTA',
    'CESSADO',
    'BENEFICIO_SUSPENSO',
    'EM_SERVICO',
    'INATIVO',
]

MatriculaFormaPagamento = Literal[
    'DINHEIRO',
    'CHEQUE',
    'CREDITO_EM_CONTA'
]

MatriculaIndicativoAdmisao = Literal[
    'NORMAL',
    'DECORRENTE_ACAO_FISCAL',
    'DECORRENTE_DECISAO_JUDICIAL',
]

CargoAcumuloCargos = Literal[
    'NAO_ACUMULAVEL',
    'PROFESSOR',
    'PROFISSIONAL_SAUDE',
    'TECNICO_CENTIFICO'
]

CargoContagemEspecial = Literal[
    'APOSENTADORIA_ESPECIAL',
    'MAGISTRADO',
    'MEMBRO_MPU',
    'MEMBRO_TCU',
    'NAO',
    'PROFESSOR',
    'PROFESSOR_ENSINO_SUPERIOR'
]

CargoGrauInstrucao = Literal[
    'ALFABETIZADO',
    'ENSINO_FUNDAMENTAL_ANOS_FINAIS',
    'ENSINO_FUNDAMENTAL_ANOS_INICIAIS',
    'ENSINO_MEDIO',
    'ENSINO_PROFISSIONALIZANTE',
    'ENSINO_SUPERIOR_SEQUENCIAL',
    'GRADUACAO_BACHARELADO',
    'GRADUACAO_LICENCIATURA',
    'GRADUACAO_TECNOLOGIA',
    'NAO_ALFABETIZADO',
    'POS_DOUTORADO_HABILITACAO',
    'POS_DOUTORADO_LIVRE_DOCENCIA',
    'POS_DOUTORADO_NOTORIO_SABER',
    'POS_GRADUACAO_DOUTORADO',
    'POS_GRADUACAO_ESPECIALIZACAO',
    'POS_GRADUACAO_HONORIS_CAUSA',
    'POS_GRADUACAO_MBA',
    'POS_GRADUACAO_MESTRADO'
]

CargoSituacaoGrauInstrucao = Literal[
    'COMPLETO',
    'INCOMPLETO'
]

CargoUnidadePagamento = Literal[
    'DIARISTA',
    'HORISTA',
    'MENSALISTA',
    'OUTROS',
    'QUINZENALISTA',
    'SEMANALISTA',
    'TAREFEIRO'
]

CampoAdicionalFormato = Literal[
    'DATE',
    'DATETIME',
    'TIME',
    'INTEGER',
    'DECIMAL',
    'TEXT',
    'TEXT_AREA',
    'CPF',
    'CNPJ',
    'EMAIL',
    'PHONE',
    'SELECT',
    'SELECT_MULTIPLE'
]



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from typing import Any, Iterable
from asyncio import create_task, gather, run
from aiohttp import ClientSession

async def _async_get_one(session: ClientSession, url: str, headers: dict[str, str], identifier: Any) -> dict[str, Any]:
    async with session.get(url.format(id=identifier), headers=headers) as response:
        return await response.json()

async def _async_get_many(url: str, headers: dict[str, str], ids: Iterable[Any]) -> list[dict[str, Any]]:
    async with ClientSession() as session:
        requests: list[Any] = []
        for identifier in ids:
            request = create_task(_async_get_one(session, url, headers, identifier))
            requests.append(request)
        responses: list[dict[str, Any]] = await gather(*requests)
        return responses

def async_get(url: str, headers: dict[str, str], ids: Iterable[Any]) -> list[dict[str, Any]]:
    """Obtém dados de várias URLs assíncronas usando uma lista de identificadores, de forma síncrona.

    ### Args:
        url (str): URL com um placeholder para os identificadores.
        headers (dict[str, str]): Cabeçalhos HTTP a serem enviados na requisição.
        ids (Iterable[Any]): lista de identificadores a serem formatados na URL.

    ### Returns:
        list[dict[str, Any]]: lista de respostas JSON das requisições.

    ### Raises:
        ValueError: Se a URL não contiver o placeholder '{id}'.

    ### Examples:
        ```python
        url: str = 'http://example.com/resource/{id}'
        headers: dict[str, str] = {'Authorization': 'Bearer token'}
        ids: list[int] = [123, 456, 789]
        
        # Reliza get requests de forma assincrona buscando pelos ids [123, 456, 789]
        data: list[dict[str, Any]] = async_get(url, headers, ids)
        ```
    """
    if '{id}' not in url:
        raise ValueError("Url informada deve conter o placeholder '{id}'")
    return run(_async_get_many(url, headers, ids))



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring, missing-class-docstring, too-many-ancestors
from marshmallow import fields as f
from ...schema import BaseSchema
from . import models as m

class MixinUuid:
    """Mixin utilizado para lidar com casos em que id cloud é um uuid"""
    cloud_uuid = f.String(data_key='id')


class SystemSchema(BaseSchema):
    class Meta(BaseSchema.Meta):
        pass


class CampoAdicionalColunaSchema(MixinUuid, SystemSchema):
    class Meta(SystemSchema.Meta):
        model = m.CampoAdicionalColuna
        exclude = ('cloud_id',)

    titulo = f.String()
    texto_ajuda = f.String()
    formato = f.String()
    variavel = f.String()
    placeholder = f.String()
    obrigatorio = f.Boolean()
    configuracoes = f.Raw()


class CampoAdicionalAgrupadorSchema(MixinUuid, SystemSchema):
    class Meta(SystemSchema.Meta):
        model = m.CampoAdicionalAgrupador
        exclude = ('cloud_id',)

    titulo = f.String()
    campos = f.List(f.Nested(CampoAdicionalColunaSchema))


class CampoAdicionalSchema(MixinUuid, SystemSchema):
    class Meta(SystemSchema.Meta):
        model = m.CampoAdicional
        exclude = ('cloud_id',)

    descricao = f.String()
    tipo = f.String()
    agrupador = f.Nested(CampoAdicionalAgrupadorSchema)


class CargoSchema(SystemSchema):
    class Meta(SystemSchema.Meta):
        model = m.Cargo

    descricao = f.String()
    campos_adicionais = f.Raw()
    codigo_e_social = f.String()
    inicio_vigencia = f.DateTime()
    paga_decimo_terceiro_salario = f.Boolean()
    grau_instrucao = f.String()
    situacao_grau_instrucao = f.String()
    contagem_especial = f.String()
    acumulo_cargos = f.String()
    dedicacao_exclusiva = f.Boolean()
    extinto = f.Boolean()
    quantidade_vagas = f.Integer()
    quantidade_vagas_pcd = f.Integer()
    requisitos_necessarios = f.String()
    atividades_desempenhadas = f.String()
    unidade_pagamento = f.String()

    historicos = f.List(f.Nested(lambda: CargoSchema(exclude=('historicos',))))



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from typing import TYPE_CHECKING, Optional, Any
from datetime import datetime
from uuid import UUID, uuid4

from sqlalchemy import UniqueConstraint as Uq
from sqlalchemy import BigInteger
from sqlalchemy.orm import Mapped
from sqlalchemy.orm import object_session
from sqlalchemy.dialects.postgresql import JSONB

from requests import get

from . import enums as en
from .system import System
from ..columns import column, relationship, standard_fk
from ..base import standard_mapping
from ..methods import batch_list

if TYPE_CHECKING:
    from requests import Response

    from ..base import Betha
    from ..schema import BaseSchema


class Identifier(System):
    """Classe identificadora que se relaciona com uma entrada de qualquer tabela."""
    integracao_id: Mapped[UUID] = column(default_factory=uuid4, sort_order=-3, unique=True, init=False)
    row_id: Mapped[int] = column(BigInteger, sort_order=-2)
    row_tablename: Mapped[str] = column(sort_order=-2)
    row_schema: Mapped[str] = column(sort_order=-2)
    validado: Mapped[bool] = column(default=False) # Define se registro foi validado com sucesso
    migrado: Mapped[bool] = column(default=False) # Define se registro foi migrado com sucesso
    situacao: Mapped[Optional[str]] = column(default=None) # Situacao de acordo com response de lote

    validacao_id: Mapped[Optional[int]] = standard_fk('validacao')
    validacao: Mapped[Optional['Validacao']] = relationship(back_populates='identificador', default=None)

    lote_id: Mapped[Optional[int]] = standard_fk('lote')
    lote: Mapped[Optional['Lote']] = relationship(back_populates='identifiers', default=None)

    @property
    def row(self) -> Optional['Betha']:
        """Busca row referenciada em identificador"""
        model = standard_mapping.get_model(self.row_schema, self.row_tablename)
        return object_session(self).get(model, self.row_id) # type: ignore

    @classmethod
    def identify(cls, row: 'Betha'):
        row.identifier = cls(row_id=row.id, row_tablename=row.__tablename__, row_schema=row.SYSTEM)

    __table_args__ = (
        Uq('row_id', 'row_tablename', 'row_schema'),
    )


class Lote(System):
    api_id: Mapped[Optional[str]] = column()
    status_code: Mapped[Optional[int]] = column()
    response: Mapped[dict[str, Any]] = column(JSONB, default_factory=dict)
    situacao: Mapped[en.LoteSituacao] = column(default='NULL')

    identifiers: Mapped[list['Identifier']] = relationship(back_populates='lote', default_factory=list)

    def json(self, schema: 'BaseSchema') -> list[dict[str, Any]]:
        """Retorna dict de json com rows em lote"""
        return [
            {'idIntegracao': str(identifier.integracao_id), 'conteudo': schema.dump(identifier.row)}
            for identifier in self.identifiers
        ]
        
    def read_response(self, response: 'Response') -> None:
        self.status_code = response.status_code
        self.response = response.json()
        
    def get_lote(self, url_lote: str, headers: dict[str, str]) -> 'Response':
        if '{id}' not in url_lote:
            raise ValueError("Url informada deve conter o placeholder '{id}'")
        return get(url_lote.format(id=self.api_id), headers=headers)

    def update(self) -> None:
        if not self.response:
            raise ValueError('Lote precisa de json response anexado para validar situacao')

        situacao: Optional[str] = self.response.get('situacao')
        if situacao in {'AGENDADO', 'EXECUTANDO'}:
            self.situacao = situacao # type: ignore
            return

        retorno: list[dict[str, Any]] = self.response.get('retorno', [])
        retorno_size = len(retorno)
        retorno_executado_size = sum(1 for row in retorno if row.get('situacao') == 'EXECUTADO')

        if retorno_executado_size == 0:
            self.situacao = 'ERRO'
        elif retorno_executado_size == retorno_size:
            self.situacao = 'SUCESSO'
        else:
            self.situacao = 'PARCIAL'
    
    @classmethod
    def make_lotes(cls, indentifiers: list[Identifier], batch_size: int = 25):
        batched_identifiers = batch_list(indentifiers, batch_size)
        return [cls(identifiers=batch) for batch in batched_identifiers]


class Validacao(System):
    timestamp: Mapped[datetime] = column(default=datetime.now(), init=False)
    identificador: Mapped['Identifier'] = relationship()

    erros: Mapped[list["ValidacaoErro"]] = relationship()


class ValidacaoErro(System):
    tipo: Mapped[en.ValidacaoErroTipo] = column()
    descricao: Mapped[Optional[str]] = column()

    validacao_id: Mapped[int] = standard_fk('validacao')
    validacao: Mapped['Validacao'] = relationship(back_populates='erros')



# -*- coding: utf-8 -*-
"""
### Methods:
    - `get_base`
    - `get_model_by_tablename`
    - `persist_enums`
    - `persist_tables`
"""

from ._get_base import basemodel
from ._async_get import async_get
from ._get_many import get_many
from ._batch_list import batch_list

__all__ = [
    'async_get',
    'basemodel',
    'get_many',
    'batch_list'
]



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from typing import TYPE_CHECKING, Type
from importlib import import_module

if TYPE_CHECKING:
    from ..base import Base

def basemodel() -> Type['Base']:
    """Retorna modelo base"""
    base = import_module('zetha.base')
    base_model: Type['Base'] = getattr(base, 'Base')
    return base_model



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring, too-many-ancestors
from typing import Optional, Any
from decimal import Decimal
from datetime import date, datetime
from collections import defaultdict
#from uuid import UUID

from sqlalchemy import BigInteger
from sqlalchemy.orm import Mapped, MappedAsDataclass
from sqlalchemy.dialects.postgresql import JSONB

#from jsonpath_ng import jsonpath, parse

from . import enums as en
from ...base import Betha
from ...columns import Num12to4, column, relationship, standard_fk
from ... import mixins as mx


class CamposAdicionais(MappedAsDataclass):
    campos_adicionais: Mapped[list[dict[str, Any]]] = column(JSONB, default_factory=list, repr=False)
    
    def flatten_campos(self) -> None:
        campos_flattened: list[dict[str, Any]] = []

        for agrupador in self.campos_adicionais:
            tipo = agrupador['tipo']
            identificador = agrupador['identificador']

            for campo in agrupador['campos']:
                campos_flattened.append({
                    'tipo': tipo,
                    'identificador': identificador,
                    'id': campo['id'],
                    'valor': campo['valor']
                })
        self.campos_adicionais = campos_flattened

    def group_campos(self) -> None:
        campos_grouped: dict[tuple[str, str], dict[str, Any]] = defaultdict(lambda: {'campos': []})

        for campo in self.campos_adicionais:
            identificador = campo['identificador']
            tipo = campo['tipo']

            key = (identificador, tipo,)

            campos_grouped[key]['identificador'] = identificador
            campos_grouped[key]['tipo'] = tipo
            campos_grouped[key]['campos'].append({
                'id': campo['id'],
                'valor': campo['valor']
            })
        self.campos_adicionais = [value for _, value in campos_grouped.items()]


class System(Betha):
    "Classe abstrata de sistema Folha"
    __abstract__ = True
    SYSTEM = 'folha'
    _BASE_URL = 'https://pessoal.betha.cloud/service-layer/v1/api/'

# TODO: Adicionar docstrings para cada modelo denotando caminho em sistema e descricao de tabela

class Ato(System):
    _PATH = 'ato'

    data_criacao: Mapped[Optional[date]] = column()
    data_publicacao: Mapped[Optional[date]] = column()
    data_resolucao: Mapped[Optional[date]] = column()
    data_vigorar: Mapped[Optional[date]] = column()
    numero_diario_oficial: Mapped[Optional[int]] = column()
    ementa: Mapped[Optional[str]] = column()
    numero_oficial: Mapped[Optional[str]] = column()

    tipo_id: Mapped[Optional[int]] = standard_fk('tipo_ato')
    tipo: Mapped['TipoAto'] = relationship()
    # fontes_divulgacao: Mapped[list['FontesDivulgacao']] = relationship(default_factory=list)


class CalculoFolha(System):
    _PATH = 'calculo-folha'

    conversao: Mapped[Optional[bool]] = column()
    data_agendamento: Mapped[Optional[date]] = column()
    data_pagamento: Mapped[Optional[date]] = column()
    tipo_vinculacao_matricula: Mapped[Optional[str]] = column()

    tipo_processamento: Mapped[Optional[en.FolhaTipoProcessamento]] = column()
    sub_tipo_processamento: Mapped[Optional[en.FolhaSubTipoProcessamento]] = column()

    folhas: Mapped[list["Folha"]] = relationship(back_populates='calculo_folha', default_factory=list)


class CampoAdicional(System, mx.Uuid):
    """Cadastro/Configuracao de campo adicional"""
    _PATH = 'campo-adicional'

    descricao: Mapped[str] = column()
    tipo: Mapped[str] = column()

    agrupador: Mapped['CampoAdicionalAgrupador'] = relationship(
        back_populates='configuracao',
        cascade='all, delete-orphan',
        default=None
    )


class CampoAdicionalAgrupador(System, mx.Uuid):
    titulo: Mapped[str] = column()

    configuracao_id: Mapped[int] = standard_fk('campo_adicional')
    configuracao: Mapped['CampoAdicional'] = relationship(back_populates='agrupador', default=None)

    campos: Mapped[list['CampoAdicionalColuna']] = relationship(
        back_populates='agrupador',
        cascade='all, delete-orphan',
        default_factory=list
    )


class CampoAdicionalColuna(System, mx.Uuid):
    ordem: Mapped[Optional[int]] = column()
    obrigatorio: Mapped[bool] = column(default=False)
    configuracoes: Mapped[Optional[dict[str, Any]]] = column(JSONB)
    placeholder: Mapped[Optional[str]] = column()
    texto_ajuda: Mapped[Optional[str]] = column()
    titulo: Mapped[str] = column()
    variavel: Mapped[str] = column()

    formato: Mapped[Optional[en.CampoAdicionalFormato]] = column()

    agrupador_id: Mapped[int] = standard_fk('campo_adicional_agrupador')
    agrupador: Mapped['CampoAdicionalAgrupador'] = relationship(back_populates='campos')


class Cargo(System, CamposAdicionais):
    """
    Tabela representa cargos em sistema Betha.
    Configurando -> Cargos e Salários -> Cargos
    """
    _PATH = 'cargo'

    id: Mapped[int] = System.id
    dedicacao_exclusiva: Mapped[Optional[bool]] = column()
    extinto: Mapped[Optional[bool]] = column()
    paga_decimo_terceiro_salario: Mapped[Optional[bool]] = column()
    inicio_vigencia: Mapped[Optional[datetime]] = column()
    quantidade_vagas: Mapped[int] = column(default=0)
    quantidade_vagas_pcd: Mapped[int] = column(default=0)
    atividades_desempenhadas: Mapped[Optional[str]] = column()
    codigo_e_social: Mapped[Optional[str]] = column()
    descricao: Mapped[Optional[str]] = column()
    requisitos_necessarios: Mapped[Optional[str]] = column()

    acumulo_cargos: Mapped[Optional[Optional[en.CargoAcumuloCargos]]] = column()
    contagem_especial: Mapped[Optional[Optional[en.CargoContagemEspecial]]] = column()
    grau_instrucao: Mapped[Optional[Optional[en.CargoGrauInstrucao]]] = column()
    situacao_grau_instrucao: Mapped[Optional[Optional[en.CargoSituacaoGrauInstrucao]]] = column()
    unidade_pagamento: Mapped[Optional[Optional[en.CargoUnidadePagamento]]] = column()

    origem_id: Mapped[Optional[int]] = standard_fk('cargo')
    origem: Mapped[Optional['Cargo']] = relationship(remote_side=[id], back_populates='historicos')
    historicos: Mapped[list['Cargo']] =  relationship(back_populates='origem', default_factory=list)

    ato_id: Mapped[int] = standard_fk('ato')
    ato: Mapped['Ato'] = relationship()

    # cbo_id: Mapped[int] = standard_fk('cbo')
    # cbo: Mapped['Cbo'] = relationship()

    # configuracao_ferias_id: Mapped[int] = standard_fk('configuracao_ferias')
    # configuracao_ferias: Mapped['ConfiguracaoFerias'] = relationship()

    # configuracao_licenca_premio_id: Mapped[int] = standard_fk('configuracao_licenca_premio')
    # configuracao_licenca_premio: Mapped['ConfiguracaoLicencaPremio'] = relationship()

    # quadro_cargos_id: Mapped[int] = standard_fk('quadro_cargos')
    # quadro_cargos: Mapped['QuadroCargos'] = relationship()

    # tipo_id: Mapped[int] = standard_fk('tipo')
    # tipo: Mapped['Tipo'] = relationship()

    # areas_atuacao: Mapped[list['AreasAtuacao']] = relationship(default_factory=list)
    # cursos: Mapped[list['Cursos']] = relationship(default_factory=list)
    # organogramas: Mapped[list['Organogramas']] = relationship(default_factory=list)
    # planos_previdencia: Mapped[list['PlanosPrevidencia']] = relationship(default_factory=list)
    # remuneracoes: Mapped[list['Remuneracoes']] = relationship(default_factory=list)
    # tipos_diaria: Mapped[list['TiposDiaria']] = relationship(default_factory=list)
    # vinculos_empregaticios: Mapped[list['VinculosEmpregaticios']] = relationship(default_factory=list)


class ConcursoProcessoSeletivo(System):
    _PATH = 'concurso-processo-seletivo'

    convocacao: Mapped[Optional[bool]] = column()
    data_avaliacao: Mapped[Optional[date]] = column()
    data_encerramento: Mapped[Optional[date]] = column()
    data_final_inscricao: Mapped[Optional[date]] = column()
    data_final_inscricao_isentos: Mapped[Optional[date]] = column()
    data_homologacao: Mapped[Optional[date]] = column()
    data_inicial_inscricao: Mapped[Optional[date]] = column()
    data_inicial_inscricao_isento: Mapped[Optional[date]] = column()
    data_prorrogacao: Mapped[Optional[date]] = column()
    data_prorrogacao_inscricao_isentos: Mapped[Optional[date]] = column()
    data_prorrogacao_validade: Mapped[Optional[date]] = column()
    data_validade: Mapped[Optional[date]] = column()
    ano_concurso: Mapped[Optional[int]] = column()
    descricao: Mapped[Optional[str]] = column()
    numero_edital: Mapped[Optional[str]] = column()
    observacao: Mapped[Optional[str]] = column()

    processo: Mapped[Optional[en.ConcursoProcessoSeletivoProcesso]] = column()
    regime_juridico: Mapped[Optional[en.ConcursoProcessoSeletivoRegimeJuridico]] = column()
    responsavel_execucao: Mapped[Optional[en.ConcursoProcessoSeletivoResponsavelExecucao]] = column()
    situacao_concurso_processo_seletivo: Mapped[Optional[en.ConcursoProcessoSeletivoSituacaoConcursoProcessoSeletivo]] = column()
    tipo_avaliacao: Mapped[Optional[en.ConcursoProcessoSeletivoTipoAvaliacao]] = column()
    tipo_processo_avaliacao: Mapped[Optional[en.ConcursoProcessoSeletivoTipoProcessoAvaliacao]] = column()
    tipo_recrutamento: Mapped[Optional[en.ConcursoProcessoSeletivoTipoRecrutamento]] = column()

    # ato_contratacao_temporaria_id: Mapped[int] = standard_fk('ato_contratacao_temporaria')
    # ato_contratacao_temporaria: Mapped['AtoContratacaoTemporaria'] = relationship()

    # ato_inscricao_id: Mapped[int] = standard_fk('ato_inscricao')
    # ato_inscricao: Mapped['AtoInscricao'] = relationship()

    # ato_inscricao_isentos_id: Mapped[int] = standard_fk('ato_inscricao_isentos')
    # ato_inscricao_isentos: Mapped['AtoInscricaoIsentos'] = relationship()

    # configuracao_recrutamento_id: Mapped[int] = standard_fk('configuracao_recrutamento')
    # configuracao_recrutamento: Mapped['ConfiguracaoRecrutamento'] = relationship()

    # contrato_id: Mapped[int] = standard_fk('contrato')
    # contrato: Mapped['Contrato'] = relationship()

    # executor_recrutamento_id: Mapped[int] = standard_fk('executor_recrutamento')
    # executor_recrutamento: Mapped['ExecutorRecrutamento'] = relationship()

    # cargos: Mapped[list['Cargos']] = relationship(default_factory=list)
    # comissoes_avaliacao: Mapped[list['ComissoesAvaliacao']] = relationship(default_factory=list)
    # concurso_processo_seletivo_avaliacoes: Mapped[list['ConcursoProcessoSeletivoAvaliacoes']] = relationship(default_factory=list)
    # editais: Mapped[list['Editais']] = relationship(default_factory=list)


class ConfiguracaoEvento(System):
    _PATH = 'configuracao-evento'

    codigo: Mapped[Optional[int]] = column(BigInteger)
    descricao: Mapped[Optional[str]] = column()
    inicio_vigencia: Mapped[Optional[datetime]] = column()
    tipo: Mapped[Optional[en.ConfiguracaoEventoTipo]] = column()
    classificacao: Mapped[Optional[en.ConfiguracaoEventoClassificacao]] = column()

    eventos: Mapped[list["FolhaEvento"]] = relationship(back_populates='configuracao', default_factory=list)
    composicao_bases: Mapped[list["FolhaComposicaoBase"]] = relationship(
        back_populates='configuracao_evento',
        default_factory=list
    )


class FolhaComposicaoBase(System):
    folha_id: Mapped[Optional[int]] = standard_fk('folha')
    base_id: Mapped[Optional[int]] = standard_fk('tipo_base')
    valor: Mapped[Decimal] = column()

    folha: Mapped["Folha"] = relationship(back_populates='composicao_bases', default=None)
    base: Mapped["TipoBase"] = relationship(default=None)

    configuracao_evento_id: Mapped[int] = standard_fk('configuracao_evento')
    configuracao_evento: Mapped["ConfiguracaoEvento"] = relationship(back_populates='composicao_bases', default=None)


class FolhaEvento(System):
    """Objeto que reprenta eventos de folha em sistema Betha"""
    tipo: Mapped[Optional[en.ConfiguracaoEventoTipo]] = column()
    referencia: Mapped[Optional[Num12to4]] = column()
    valor: Mapped[Optional[Decimal]] = column()
    classificacao: Mapped[Optional[en.ConfiguracaoEventoClassificacao]] = column()
    lancamento_variavel: Mapped[bool] = column(default=False)

    folha_id: Mapped[int] = standard_fk('folha')
    folha: Mapped["Folha"] = relationship(back_populates='eventos', default=None)

    configuracao_id: Mapped[int] = standard_fk('configuracao_evento')
    configuracao: Mapped["ConfiguracaoEvento"] = relationship(back_populates='eventos', default=None)


class Folha(System):
    """Objeto que representa folhas/remuneracoes em sistema Betha."""
    _PATH = 'folha'

    tipo_processamento: Mapped[Optional[en.FolhaTipoProcessamento]] = column()
    sub_tipo_processamento: Mapped[Optional[en.FolhaSubTipoProcessamento]] = column()
    competencia: Mapped[Optional[str]] = column()
    folha_pagamento: Mapped[Optional[bool]] = column()
    total_bruto: Mapped[Optional[Decimal]] = column()
    total_desconto: Mapped[Optional[Decimal]] = column()
    total_liquido: Mapped[Optional[Decimal]] = column()
    data_fechamento: Mapped[Optional[date]] = column()
    data_pagamento: Mapped[Optional[date]] = column()
    data_liberacao: Mapped[Optional[date]] = column()
    data_calculo: Mapped[Optional[date]] = column()
    situacao: Mapped[Optional[en.FolhaSituacao]] = column()
    identificador_folha_de_pagamento: Mapped[Optional[str]] = column()
    pagamento_anterior: Mapped[Optional[int]] = column(BigInteger)
    pagamento_anterior_parcela: Mapped[Optional[int]] = column(BigInteger)
    quantidade_meses_rra: Mapped[Optional[Decimal]] = column()
    enviar_remuneracoes_e_social: Mapped[bool] = column(default=False)
    enviar_pagamentos_e_social: Mapped[bool] = column(default=False)

    matricula_id: Mapped[int] = standard_fk('matricula')
    matricula: Mapped['Matricula'] = relationship(back_populates='folhas', default=None)

    calculo_folha_id: Mapped[int] = standard_fk('calculo_folha')
    calculo_folha: Mapped['CalculoFolha'] = relationship(back_populates='folhas', default=None)

    eventos: Mapped[list['FolhaEvento']] = relationship(back_populates='folha', default=None)
    composicao_bases: Mapped[list['FolhaComposicaoBase']] = relationship(back_populates='folha', default_factory=list)


class GrupoFuncional(System):
    pass


class Matricula(System):
    area_atuacao_alterada: Mapped[Optional[bool]] = column()
    beneficio_concedido_decisao_judicial: Mapped[Optional[bool]] = column()
    cargo_alterado: Mapped[Optional[bool]] = column()
    cargo_comissionado_adicionado: Mapped[Optional[bool]] = column()
    clausula_assecuratoria_contrato_temporario: Mapped[Optional[bool]] = column()
    compensa_horas: Mapped[Optional[bool]] = column()
    conselheiro_tutelar: Mapped[Optional[bool]] = column()
    controla_horas_manual: Mapped[Optional[bool]] = column()
    controla_per_aquis_ferias_antes_admissao: Mapped[Optional[bool]] = column()
    enviar_esocial: Mapped[Optional[bool]] = column()
    estagio_obrigatorio: Mapped[Optional[bool]] = column()
    gera_registro_preliminar: Mapped[Optional[bool]] = column()
    jornada_parcial: Mapped[Optional[bool]] = column()
    ocupa_vaga: Mapped[Optional[bool]] = column()
    ocupa_vaga_comissionado: Mapped[Optional[bool]] = column()
    optante_fgts: Mapped[Optional[bool]] = column()
    outra_origem: Mapped[Optional[bool]] = column()
    possui_seguro_vida: Mapped[Optional[bool]] = column()
    previdencia_federal: Mapped[Optional[bool]] = column()
    primeiro_emprego: Mapped[Optional[bool]] = column()
    profissional_saude_seguranca_publica: Mapped[Optional[bool]] = column()
    recebe_abono_permanencia: Mapped[Optional[bool]] = column()
    recebe_decimo_terceiro: Mapped[Optional[bool]] = column()
    registra_ponto: Mapped[Optional[bool]] = column()
    salario_alterado: Mapped[Optional[bool]] = column()
    salario_comissionado_alterado: Mapped[Optional[bool]] = column()
    data_admissao_origem: Mapped[Optional[date]] = column()
    data_admissao_retificada_proc_trab: Mapped[Optional[date]] = column()
    data_agendamento_rescisao: Mapped[Optional[date]] = column()
    data_alteracao_salario_esocial: Mapped[Optional[date]] = column()
    data_base: Mapped[Optional[date]] = column()
    data_final: Mapped[Optional[date]] = column()
    data_inicial_periodo_aquisitivo_ferias_antes_admissao: Mapped[Optional[date]] = column()
    data_inicio_contrato: Mapped[Optional[date]] = column()
    data_nomeacao: Mapped[Optional[date]] = column()
    data_obito: Mapped[Optional[date]] = column()
    data_opcao: Mapped[Optional[date]] = column()
    data_posse: Mapped[Optional[date]] = column()
    data_prorrogacao: Mapped[Optional[date]] = column()
    data_prorrogacao_contrato_temporario: Mapped[Optional[date]] = column()
    data_saida_cargo_comissionado: Mapped[Optional[date]] = column()
    data_termino_contrato_temporario: Mapped[Optional[date]] = column()
    data_transferencia: Mapped[Optional[date]] = column()
    inicio_abono_permanencia: Mapped[Optional[date]] = column()
    inicio_turno_corrido: Mapped[Optional[date]] = column()
    inicio_vigencia: Mapped[Optional[datetime]] = column()
    quantidade_horas_mes: Mapped[Optional[Decimal]] = column()
    rendimento_mensal: Mapped[Optional[Decimal]] = column()
    salario_comissionado: Mapped[Optional[Decimal]] = column()
    entidade_origem: Mapped[Optional[int]] = column()
    quantidade_horas_semana: Mapped[Optional[int]] = column()
    cnpj_entidade_qualificadora: Mapped[Optional[str]] = column()
    conta_fgts: Mapped[Optional[str]] = column()
    cpf_matricula_origem: Mapped[Optional[str]] = column()
    descricao_beneficio: Mapped[Optional[str]] = column()
    descricao_salario: Mapped[Optional[str]] = column()
    e_social: Mapped[Optional[str]] = column()
    matricula_empresa_origem: Mapped[Optional[str]] = column()
    motivo_contrato_temporario: Mapped[Optional[str]] = column()
    n_processo_trabalhista: Mapped[Optional[str]] = column()
    numero_apolice_seguro_vida: Mapped[Optional[str]] = column()
    numero_beneficio: Mapped[Optional[str]] = column()
    numero_beneficio_anterior: Mapped[Optional[str]] = column()
    numero_cartao_ponto: Mapped[Optional[str]] = column()
    numero_contrato: Mapped[Optional[str]] = column()
    objetivo: Mapped[Optional[str]] = column()
    observacao: Mapped[Optional[str]] = column()
    observacao_contrato: Mapped[Optional[str]] = column()

    # controle_jornada: Mapped['MatriculaControleJornada'] = column()
    # duracao_beneficio: Mapped['MatriculaDuracaoBeneficio'] = column()
    # formacao_fase: Mapped['MatriculaFormacaoFase'] = column()
    # formacao_periodo: Mapped['MatriculaFormacaoPeriodo'] = column()
    # forma_pagamento: Mapped['MatriculaFormaPagamento'] = column()
    # hipotese_legal_contrato_temporario: Mapped['MatriculaHipoteseLegalContratoTemporario'] = column()
    # indicativo_admissao: Mapped['MatriculaIndicativoAdmissao'] = column()
    # indicativo_provimento: Mapped['MatriculaIndicativoProvimento'] = column()
    # motivo_inicio_beneficio: Mapped['MatriculaMotivoInicioBeneficio'] = column()
    # natureza_atividade: Mapped['MatriculaNaturezaAtividade'] = column()
    # ocorrencia_sefip: Mapped['MatriculaOcorrenciaSefip'] = column()
    # origem_concurso: Mapped['MatriculaOrigemConcurso'] = column()
    # origem_salario: Mapped['MatriculaOrigemSalario'] = column()
    # prazo_determinado_contrato_temporario: Mapped['MatriculaPrazoDeterminadoContratoTemporario'] = column()
    # situacao: Mapped['MatriculaSituacao'] = column()
    # situacao_beneficio: Mapped['MatriculaSituacaoBeneficio'] = column()
    # tempo_aposentadoria: Mapped['MatriculaTempoAposentadoria'] = column()
    # tipo: Mapped['MatriculaTipo'] = column()
    # tipo_admissao: Mapped['MatriculaTipoAdmissao'] = column()
    # tipo_contratacao_aprendiz: Mapped['MatriculaTipoContratacaoAprendiz'] = column()
    # tipo_contrato_parcial: Mapped['MatriculaTipoContratoParcial'] = column()
    # tipo_de_beneficio: Mapped['MatriculaTipoDeBeneficio'] = column()
    # tipo_inclusao_contrato_temporario: Mapped['MatriculaTipoInclusaoContratoTemporario'] = column()
    # tipo_provimento: Mapped['MatriculaTipoProvimento'] = column()
    # unidade_pagamento: Mapped['MatriculaUnidadePagamento'] = column()

    # ato_contrato_id: Mapped[int] = standard_fk('ato')
    # ato_contrato: Mapped['Ato'] = relationship()

    # ato_alteracao_salario_id: Mapped[int] = standard_fk('ato')
    # ato_alteracao_salario: Mapped['Ato'] = relationship()

    # conta_bancaria_pagamento_id: Mapped[int] = standard_fk('conta_bancaria')
    # conta_bancaria_pagamento: Mapped['ContaBancaria'] = relationship()

    # pessoa_id: Mapped[int] = standard_fk('pessoa')
    # pessoa: Mapped['Pessoa'] = relationship()

    # grupo_funcional_id: Mapped[int] = standard_fk('grupo_funcional')
    # grupo_funcional: Mapped['GrupoFuncional'] = relationship()

    # jornada_trabalho_id: Mapped[int] = standard_fk('jornada_trabalho')
    # jornada_trabalho: Mapped['JornadaTrabalho'] = relationship()

    # motivo_alteracao_salario_id: Mapped[int] = standard_fk('motivo_alteracao_salario')
    # motivo_alteracao_salario: Mapped['MotivoAlteracaoSalario'] = relationship()

    # organograma_id: Mapped[int] = standard_fk('organograma')
    # organograma: Mapped['Organograma'] = relationship()

    # processo_trabalhista_id: Mapped[int] = standard_fk('processo_trabalhista')
    # processo_trabalhista: Mapped['ProcessoTrabalhista'] = relationship()

    # rescisao_id: Mapped[int] = standard_fk('rescisao')
    # rescisao: Mapped['Rescisao'] = relationship()

    # vinculo_empregaticio_id: Mapped[int] = standard_fk('vinculo_empregaticio')
    # vinculo_empregaticio: Mapped['VinculoEmpregaticio'] = relationship()

    folhas: Mapped[list['Folha']] = relationship(default_factory=list)
    # funcoes_gratificadas: Mapped[list['FuncoesGratificadas']] = relationship(default_factory=list)
    # historicos: Mapped[list['Historicos']] = relationship(default_factory=list)
    # lotacoes_fisicas: Mapped[list['LotacoesFisicas']] = relationship(default_factory=list)
    # matricula_licencas_premio: Mapped[list['MatriculaLicencasPremio']] = relationship(default_factory=list)
    # matriculas_adicionais: Mapped[list['MatriculasAdicionais']] = relationship(default_factory=list)
    # previdencias: Mapped[list['Previdencias']] = relationship(default_factory=list)
    # reintegracoes: Mapped[list['Reintegracoes']] = relationship(default_factory=list)
    # responsaveis: Mapped[list['Responsaveis']] = relationship(default_factory=list)


class Pessoa(System):
    _PATH = 'pessoa'


class TipoAto(System):
    _PATH = 'tipo-ato'
    descricao: Mapped[Optional[str]] = column()
    classificacao: Mapped[Optional[en.TipoAtoClassificacao]] = column()


class TipoBase(System):
    _PATH = 'base'

    descricao: Mapped[Optional[str]] = column()
    sigla: Mapped[Optional[str]] = column()
    classificacao_base_calculo: Mapped[Optional[en.TipoBaseClassificacao]] = column()



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from typing import Any, Iterable
from requests import get # type: ignore

def get_many(url: str, headers: dict[str, str], ids: Iterable[Any]) -> list[dict[str, Any]]:
    if '{id}' not in url:
        raise ValueError("Url informada deve conter o placeholder '{id}'")

    response: list[dict[str, Any]] = []
    for identifier in ids:
        request = get(url.format(id=identifier), headers=headers) # pylint: disable=missing-timeout
        response.append(request.json())
    return response



# -*- coding: utf-8 -*-
# pylint: disable=missing-module-docstring
from .models import (
    Identifier,
    Lote,
    Validacao,
    ValidacaoErro
)

__all__ = [
    'Identifier',
    'Lote',
    'Validacao',
    'ValidacaoErro'
]



# -*- coding: utf-8 -*-
from typing import Literal

LoteSituacao = Literal[
    'NULL',
    'AGENDADO',
    'EXECUTANDO',
    'SUCESSO',
    'ERRO',
    'PARCIAL'
]

ValidacaoErroTipo = Literal[
    'CAMPO_REQUERIDO_NULO',
    'VALOR_MONETARIO_NEGATIVO'
]



from setuptools import setup, find_packages # type: ignore

setup(
    name='zetha',
    version='0.1.0',
    author='Bruno Moretti',
    url='https://github.com/StafSis/Zetha',
    packages=find_packages(where='zetha'),
    package_dir={'': 'zetha'},
)



# mypy: disable-error-code="name-defined, valid-type"
def batch_list[T](items: list[T], batch_size: int) -> list[list[T]]:
    """Divide uma sequência em sublistas de tamanho fixo. Similar a metodo collate de Groovy.

    Esta função divide uma lista em sublistas (ou batches) de tamanho fixo especificado. 
    Se a lista não puder ser dividida igualmente, a última sublista conterá os elementos restantes.

    ### Args:
        items (lista[T]): A lista de elementos a ser dividida.
        batch_size (int): O tamanho de cada sublista.

    ### Returns:
        list[list[T]]: Uma lista de sublistas, onde cada sublista tem no máximo `batch_size` elementos.

    ### Examples:
        ```python
        >>> batch_list([1, 2, 3, 4, 5], 2)
        [[1, 2], [3, 4], [5]]

        >>> batch_list([1, 2, 3], 3)
        [[1, 2, 3]]
        ```

    ### Notes:
        - Se `batch_size` for maior que o tamanho da `sequence`,
        a função retornará uma lista com uma única sublista contendo toda a `sequence`.
        - Se `batch_size` for menor ou igual a 0, a função retornará uma lista vazia.
    """
    return [items[i:i + batch_size] for i in range(0, len(items), batch_size)]



"""
### Models:
    - `Ato`
    - `CalculoFolha`
    - `CampoAdicional`
    - `CampoAdicionalAgrupador`
    - `CampoAdicionalColuna`
    - `Cargo`
    - `ConcursoProcessoSeletivo`
    - `ConfiguracaoEvento`
    - `FolhaComposicaoBase`
    - `FolhaEvento`
    - `Folha`
    - `GrupoFuncional`
    - `Matricula`
    - `Pessoa`
    - `TipoAto`
    - `TipoBase`
"""

from .models import (
    Ato,
    CalculoFolha,
    CampoAdicional,
    CampoAdicionalAgrupador,
    CampoAdicionalColuna,
    Cargo,
    ConcursoProcessoSeletivo,
    ConfiguracaoEvento,
    FolhaComposicaoBase,
    FolhaEvento,
    Folha,
    GrupoFuncional,
    Matricula,
    Pessoa,
    TipoAto,
    TipoBase,
)

from .schemas import *

__all__ = [
    'Ato',
    'CalculoFolha',
    'CampoAdicional',
    'CampoAdicionalAgrupador',
    'CampoAdicionalColuna',
    'Cargo',
    'ConcursoProcessoSeletivo',
    'ConfiguracaoEvento',
    'FolhaComposicaoBase',
    'FolhaEvento',
    'Folha',
    'GrupoFuncional',
    'Matricula',
    'Pessoa',
    'TipoAto',
    'TipoBase',
]
